@@ -12,10 +12,45 @@ from .anchor_explanation import AnchorExplanation
 
 if TYPE_CHECKING:
     import spacy
-
 logger = logging.getLogger(__name__)
 
 
+def _load_spacy_lexeme_prob(nlp: 'spacy.language.Language'):
+    """
+    This utility function loads the `lexeme_prob` table for a spacy model if it is not present.
+    This is required to enable support for different spacy versions.
+    """
+    import spacy
+    SPACY_VERSION = spacy.__version__.split('.')
+    MAJOR, MINOR = int(SPACY_VERSION[0]), int(SPACY_VERSION[1])
+
+    if MAJOR == 2:
+        if MINOR < 3:
+            return nlp
+        elif MINOR == 3:
+            # spacy 2.3.0 moved lexeme_prob into a different package `spacy_lookups_data`
+            # https://github.com/explosion/spaCy/issues/5638
+            try:
+                table = nlp.vocab.lookups_extra.get_table('lexeme_prob')
+                # remove the default empty table
+                if table == dict():
+                    nlp.vocab.lookups_extra.remove_table('lexeme_prob')
+            except KeyError:
+                pass
+            finally:
+                # access the `prob` of any word to load the full table
+                assert nlp.vocab["a"].prob != -20.0, f"Failed to load the `lexeme_prob` table for model {nlp}"
+    elif MAJOR >= 3:
+        # in spacy 3.x we need to manually add the tables
+        # https://github.com/explosion/spaCy/discussions/6388#discussioncomment-331096
+        if 'lexeme_prob' not in nlp.vocab.lookups.tables:
+            from spacy.lookups import load_lookups
+            lookups = load_lookups(nlp.lang, ['lexeme_prob'])
+            nlp.vocab.lookups.add_table('lexeme_prob', lookups.get_table('lexeme_prob'))
+
+    return nlp
+
+
 class Neighbors(object):
 
     def __init__(self, nlp_obj: 'spacy.language.Language', n_similar: int = 500, w_prob: float = -15.) -> None:
@@ -31,15 +66,7 @@ class Neighbors(object):
         w_prob
             Smoothed log probability estimate of token's type.
         """
-
         self.nlp = nlp_obj
-        # spacy 2.3.0 moved lexeme_prob into a different package `spacy_lookups_data`
-        # https://github.com/explosion/spaCy/issues/5638
-        try:
-            self.nlp.vocab.lookups_extra.remove_table('lexeme_prob')
-        except AttributeError:
-            pass
-
         self.w_prob = w_prob
         # list with spaCy lexemes in vocabulary
         # first if statement is a workaround due to some missing keys in models:
@@ -114,7 +141,7 @@ class AnchorText(Explainer):
         super().__init__(meta=copy.deepcopy(DEFAULT_META_ANCHOR))
         np.random.seed(seed)
 
-        self.nlp = nlp
+        self.nlp = _load_spacy_lexeme_prob(nlp)
 
         # check if predictor returns predicted class or prediction probabilities for each class
         # if needed adjust predictor so it returns the predicted class
@@ -530,7 +557,7 @@ class AnchorText(Explainer):
         Returns
         -------
         explanation
-            Dictionary containing the anchor explaining the instance with additional metadata.
+            `Explanation` object containing the anchor explaining the instance with additional metadata as attributes.
         """
         # get params for storage in meta
         params = locals()
